//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef POSTFX_HPP
#define POSTFX_HPP

#include "Drawable.hpp"

#include "ClassWithProtectedRc.hpp"
//------------------------------------------------------------------------------
namespace En
{
/// LUA_HEADER_COPY_ON ///
/** A post effect is like a texture shader. It is used to add effects to the final image (add blur, fade, colorize, lens effects...).
  * The language used is defined by SFML.
  * You should use Group::CreatePostFX to create a PostFX object.
  */
/// LUA_HEADER_COPY_OFF ///

class PostFX : public DrawableSF, public sf::PostFX, public ClassWithProtectedRcMultiple
{
public:
    PostFX(Group *parent, const std::string &name);
    static PostFX *Create(Group *parent, const std::string &name) {return new PostFX(parent, name);}

    /// LUA_EXPORT_ON(PostFX, Drawable, ProtectedPtr<Object>) ///
    // From sf::PostFX
    bool LoadFromFile(const std::string& Filename);
    /** Add a parameter named \\a name to the PostFX. */
    //LUA// void SetParameter(const std::string& Name, float X);
    /** Add a Vector2 parameter named \\a name to the PostFX. */
    //LUA// void SetParameter(const std::string& Name, float X, float Y);
    /** Add a Vector3 parameter named \\a name to the PostFX. */
    //LUA// void SetParameter(const std::string& Name, float X, float Y, float Z);
    /** Add a Vector4 parameter named \\a name to the PostFX. */
    //LUA// void SetParameter(const std::string& Name, float X, float Y, float Z, float W);
    /** Add a Texture parameter named \\a name to the PostFX. */
    void SetTexture(const std::string& name, Texture* image);
    /** Return true if the current system can use PostFX. */
    //LUA// static bool CanUsePostFX();

    /// LUA_EXPORT_OFF ///

    // No Lua
    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual PostFX *GetRawCloneToGroup(Group *newParent) const {PostFX *obj = new PostFX(*this); obj->RegisterClone(newParent); return obj;}

    // Acces to position & rotation
    Vector2D GetPosition() const {return DrawableSF::GetPosition();}
    virtual void SetPosition(const Vector2D &pos) {DrawableSF::SetPosition(pos);}
    float GetRotation() const {return DrawableSF::GetRotation();}
    virtual void SetRotation(float angle) {DrawableSF::SetRotation(angle);}
    void Rotate(float angle) {DrawableSF::Rotate(angle);}
    void Move(const Vector2D &mvt) {DrawableSF::Move(mvt);}

    // Render
    virtual void Render(sf::RenderTarget &App);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_DRAWABLE_POSTFX)

protected:
};
//------------------------------------------------------------------------------
} // namespace En

#endif // POSTFX_HPP
