//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "SoundManager.hpp"
#include "Utils.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
SoundManager::SoundManager()
{

}
//------------------------------------------------------------------------------
SoundManager::~SoundManager()
{
    ClearAllSounds();
}
//------------------------------------------------------------------------------
void SoundManager::ClearAllSounds()
{
    for(SndTblType::iterator it = m_sndTbl.begin(); it != m_sndTbl.end(); it++ )
    {
        delete *it;
    }
    m_sndTbl.clear();
}
//------------------------------------------------------------------------------
/*void SoundManager::PlayProtectedSound(SndTblType::iterator it)
{
    sound
    (*it)->SetBuffer(*sound);
    (*it)->Play();
}*/
//------------------------------------------------------------------------------
ProtectedPtr<SoundPlayer> SoundManager::PlaySound(Sound &sound)
{
    SndTblType::iterator it;
    for(it = m_sndTbl.begin(); it != m_sndTbl.end(); it++ )
    {
        if((*it)->GetStatus() != sf::Sound::Playing)
        {
            (*it)->SetSound(&sound);
            (*it)->Play();
            break;
        }
    }

    if(it == m_sndTbl.end()) // Pas de libre trouvé
    {
        it = m_sndTbl.insert(new SoundPlayer(this, &sound)).first;
        (*it)->Play();
        //m_sndTbl.push_back(new SoundPlayer(this, sound));
        //(m_sndTbl.back())->Play();
    }
    else // Clean
    {
        for(SndTblType::iterator it2 = m_sndTbl.begin(); it2 != m_sndTbl.end();)
        {
            if((*it2)->GetStatus() != sf::Sound::Playing)
            {
                SndTblType::iterator nextIt = it2;
                nextIt++;

                delete *it2;
                m_sndTbl.erase(it2);
                it2 = nextIt;
            }
            else
            {
                it2++;
            }
        }
    }

    return *it;
}
//------------------------------------------------------------------------------
/** Stop all the sound currently playing in this SoundManager */
void SoundManager::StopAllSounds()
{
    ClearAllSounds();
}
//------------------------------------------------------------------------------
/** Pause all sounds (when the screen lost focus) */
void SoundManager::PauseAllSounds()
{
    // Sound
    for(SndTblType::iterator it = m_sndTbl.begin(); it != m_sndTbl.end(); it++ )
    {
        if((*it)->GetStatus() == sf::Sound::Playing)
        {
            (*it)->Pause();
        }
    }
}
//------------------------------------------------------------------------------
/** Resume all sounds (when the screen gain focus) */
void SoundManager::ResumeAllSounds()
{
    // Sound
    for(SndTblType::iterator it = m_sndTbl.begin(); it != m_sndTbl.end(); it++ )
    {
        if((*it)->GetStatus() == sf::Sound::Paused) // Pas les stoped
        {
            (*it)->Play();
        }
    }
}
//------------------------------------------------------------------------------
void SoundManager::UnregisterSound(SoundPlayer *sndToUnregister)
{
    //SndTblType::iterator it = std::find(m_sndTbl.begin(), m_sndTbl.end(), sndToUnregister);
    SndTblType::iterator it = m_sndTbl.find(sndToUnregister);

    if(it != m_sndTbl.end())
    {
        m_sndTbl.erase(it);
    }
    else
    {
        MY_THROW("SoundManager::UnregisterSound() : SoundPlayer not found !")
    }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
/** Create a sound with the given buffer */
SoundPlayer::SoundPlayer(SoundManager *parentSndManager, En::Sound *sound) : sf::Sound(), m_parentSndManager(parentSndManager), m_currentSnd(NULL)
{
    SetSound(sound);
}
//------------------------------------------------------------------------------
/** Destructor. Will unregister the sound */
SoundPlayer::~SoundPlayer()
{
    if(m_currentSnd != NULL)
    {
        m_currentSnd->RemoveSoundBufferCallback(this);
    }
}
//------------------------------------------------------------------------------
/** Set the buffer for that sound and protect it */
void SoundPlayer::SetSound(En::Sound *buffer)
{
    if(m_currentSnd != NULL)
    {
        m_currentSnd->RemoveSoundBufferCallback(this);
    }

    m_currentSnd = buffer;
    m_currentSnd->AddSoundBufferCallback(this);

    SetBuffer(*m_currentSnd);
}
//------------------------------------------------------------------------------
/** Get the buffer used for that sound */
ProtectedPtr<En::Sound> SoundPlayer::GetSound()
{
    return m_currentSnd;
}
//------------------------------------------------------------------------------
void SoundPlayer::SetBuffer(const sf::SoundBuffer &Buffer)
{
    sf::Sound::SetBuffer(Buffer);
}
//------------------------------------------------------------------------------

/// LUA_EXPORT_ON(SoundPlayer, ProtectedPtr<SoundPlayer>) ///
/** Stop and destoy this SoundPlayer */
void SoundPlayer::Stop()
{
    m_parentSndManager->UnregisterSound(this);
    delete this;
}
//------------------------------------------------------------------------------

} // namespace En
//------------------------------------------------------------------------------
