------------------------------------------------------------
-- Missile 1
------------------------------------------------------------

-- Fns
function missile1_Create(toGroup, forcedPos, enemyLevel)
	if forcedPos == nil then
		forcedPos = Vector2D(math.random() * screenSize.x, 0)
	end
	
	local enemy = stockEnemy1:CloneToGroup(toGroup)
	enemy:SetPosition(forcedPos)
	
	-- Speed
	enemy:SetSpeed(enemy:GetSpeed() * (0.8 + math.random() * 0.4)) -- Speed +- 40%
	
	-- Level
	local texTbl = missileTexTbl;
	
	enemyLevel = math.min(enemyLevel, #texTbl)
	enemyLevel = math.max(enemyLevel, 1)
	
	enemy:FindByNameSprite("spr"):SetImage(texTbl[enemyLevel])
	enemy:SetUserData("damage", 20 + 10 * enemyLevel)
	enemy:SetUserData("points", enemyLevel)
	
	-- Return
	return enemy
end

function missile1_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	enemyGrp:SetUserData("life", enemyGrp:GetUserData("life") - bulletGrp:GetUserData("damage"))

	if enemyGrp:GetUserData("life") <= 0 then -- If the enemy is dead
		-- Enemy fn
		-- Explosion
		CreateExplosion(hitPos, math.pi * 3/5, bulletGrp:GetSpeed())

		-- Play Snd
		gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
		
		-- Points
		score = score + enemyGrp:GetUserData("points")
		
		enemyGrp:Destroy() -- We destroy the group
	else -- The enemy is style alive
		gameScreen:PlaySound(sndEnemyResist)
	end
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function missile1_OnShieldShock(phShield, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	
	-- Damage
	ChangeLife(- enemyGrp:GetUserData("damage"))
	CreateShieldImpact(hitPos.x)
	
	-- Explosion
	CreateExplosion(hitPos, math.pi, Vector2D(0, -1))
	gameScreen:PlaySound(sndShieldResist)
	
	-- We destroy the group
	enemyGrp:Destroy()
end

-- Save
addToSpawnList(function(toGroup, forcedPos) missile1_Create(toGroup, forcedPos, 1); end, 1, 1.5)
addToSpawnList(function(toGroup, forcedPos) missile1_Create(toGroup, forcedPos, 2); end, 3, 1.5)
addToSpawnList(function(toGroup, forcedPos) missile1_Create(toGroup, forcedPos, 3); end, 12, 1)
addToSpawnList(function(toGroup, forcedPos) missile1_Create(toGroup, forcedPos, 3); end, 17, 1.5)

-- Missile
stockEnemy1 = stockScreen:CreateGroup("enemy1")
stockEnemy1:SetSpeed(Vector2D(0, 120))
stockEnemy1:SetUserData("life", 20)
stockEnemy1:SetUserData("enemyType", "missile1")

-- Functions
stockEnemy1:SetUserData("OnHitFn", missile1_OnHit)
stockEnemy1:SetUserData("OnShieldShockFn", missile1_OnShieldShock)

-- Sprite
tmpSprite = stockEnemy1:CreateSprite("spr")
tmpSprite:SetImage(missileTexTbl[1])
tmpSprite.y = - tmpSprite:GetSize().y
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
--stockEnemy1:CreatePhTriangle(phGroupEnemies, "ph", Vector2D(- tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y), Vector2D(tmpSprite:GetSize().x / 2, - tmpSprite:GetSize().y), Vector2D(0, 0))
local ph = stockEnemy1:CreatePhRect(phGroupEnemies, "ph", tmpSprite:GetSize())
ph:SetPosition(Vector2D(- tmpSprite:GetSize().x / 2,  - tmpSprite:GetSize().y))

-- Reacteur --
model = ParticleModel(0.3, 0.8)
model:AddMutableParam("alpha", 0.7, 0)
model:AddMutableRandomParam("blue", 0.3, 0.5, 0, 0.1)
model:AddMutableRandomParam("red", 0.8, 1, 0.7, 1)
model:AddMutableRandomParam("green", 0.8, 1, 0, 0.2)
model:AddMutableRandomParam("size", 0.7, 1, 1.5, 2)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 4)
emitter:SetAngles(3.14 / 4, 0)
emitter:SetDirection(Vector2D(0, -1))

-- Particle
particleObj = stockEnemy1:CreateParticleSource("particle", model, 24)
particleObj:MakeRenderImages(texParticlePoint, Vector2D(40, 40), true)
particleObj:SetEmitter(emitter)
particleObj:SetFlow(30)
particleObj:SetForce(30, 100)
particleObj:SetFriction(3)
particleObj:SetGravity(Vector2D(0, -15))
particleObj.position = Vector2D(-1, -91)
particleObj.zPos = -100
